package com.toolcafe.gwtstore.shared.model;

/**
 * Title:        Address object
 * Description:
 * Copyright:    Copyright (c)
 * Account:      EJBPeople, Inc.
 * @author Rick Horowitz
 * @version 1.0
 */

import com.google.code.twig.annotation.Entity;
import com.toolcafe.gwt.client.exception.ToolCafeInputException;
import com.toolcafe.gwt.client.exception.ToolCafeValidateException;
import com.toolcafe.gwt.client.util.ClientUtil;
import com.toolcafe.gwt.shared.model.ICopyable;
import com.toolcafe.gwt.shared.model.ISearchSource;
import com.toolcafe.gwt.shared.model.IStorable;

@Entity(allocateIdsBy=10, kind="Address")
public class Address implements ICopyable, ISearchSource, IStorable {

	// ------------------------------------------------------- Statics
	/**
	 * Serializable version id When evolving this class with compatible changes,
	 * copy this version id to the new class
	 */
	static final long serialVersionUID = 100L;

	/** Keys used to interact with fields in GWT client */
	public static enum ADDRESS_KEYS {
		ADDRESS_1, ADDRESS_2, CITY, STATE, COUNTRY, POSTAL_CODE
	}

	// ----------------------------------------------------- Instance variables

	/**
	 * Unique id of the object - used as the object's primary key in the
	 * database
	 */
	private String key;
	/** Version nbr for this entity in the database */
	private Long version;
	/** 1st ADDRESS line */
	private String address1;
	/** 2nd ADDRESS line */
	private String address2;
	/** City */
	private String city;
	/** State - contains the entire state name */
	private String state;
	/** Country name */
	private String country;
	/** Postal code */
	private String postalCode;
	/** The Person owner of this ADDRESS
	@SearchableCascading(cascade = Cascade.ALL) 
	private Person person; */

	// ----------------------------------------------------- Constructors

	public Address() {
	}

	/**
	 * Construct the ADDRESS
	 * 
	 * @param address1
	 *            Address line 1
	 * @param address2
	 *            Address line 2
	 * @param city
	 *            The city
	 * @param state
	 *            The state
	 * @param country
	 *            The country
	 * @param postalCode
	 *            Postal/ZIP code
	 */
	public Address(String address1, String address2, String city, String state,
			String country, String postalCode) {

		this.address1 = address1;
		this.address2 = address2;
		this.city = city;
		this.state = state;
		this.country = country;
		this.postalCode = postalCode;
	}
	
	public String getKey() {
		return key;
	}

	public void setKey(String key) {
		this.key = key;
	}

	public Long getVersion() {
		return version;
	}

	public void setVersion(Long version) {
		this.version = version;
	}

	@Override
	public void loadObject() {
	}

	/**
	 * Validate the ADDRESS object
	 */
	public void validate() throws ToolCafeValidateException {
		if (address1 == null)
			throw new ToolCafeValidateException("Address must be specified",
					null);
		if (city == null)
			throw new ToolCafeValidateException("City must be specified", null);
		if (postalCode == null)
			throw new ToolCafeValidateException("Invalid postal code", null);
		if (state == null)
			throw new ToolCafeValidateException("Invalid state", null);
		if (country == null)
			throw new ToolCafeValidateException("Invalid country", null);
	}

	/**
	 * Require an overridden equals() method
	 */
	@Override
	public boolean equals(Object obj) {
		// (TBD) - need to improve equals processing to take into account
		// differences
		// in how addresses are typed in.
		if (!(obj instanceof Address))
			return false;
		Address that = (Address) obj;
		ClientUtil clientUtil = ClientUtil.getInstance();
		if (!clientUtil.objEquals(getAddress1(), that.getAddress1()))
			return false;
		if (!clientUtil.objEquals(getAddress2(), that.getAddress2()))
			return false;
		if (!clientUtil.objEquals(getCity(), that.getCity()))
			return false;
		if (!clientUtil.objEquals(getState(), that.getState()))
			return false;
		if (!clientUtil.objEquals(getCountry(), that.getCountry()))
			return false;
		if (!clientUtil.objEquals(getPostalCode(), that.getPostalCode()))
			return false;
		return true;
	}

	/**
	 * Copy from source object to this object.
	 * 
	 * @param source
	 *            Address object to copy from
	 */
	public void copyFrom(Object address, boolean deepCopy) throws ToolCafeInputException {
		if (!(address instanceof Address))
			throw new ToolCafeInputException(
					"Copy attempt failed. Requires Address "
							+ "object as source.", null);
		Address source = (Address) address;
		setAddress1(source.getAddress1());
		setAddress2(source.getAddress2());
		setCity(source.getCity());
		setState(source.getState());
		setCountry(source.getCountry());
		setPostalCode(source.getPostalCode());
	}

	/**
	 * Get a short (1 line) Address string
	 * 
	 * @return Short Address string
	 */
	public String getShortAddrString() {
		StringBuffer sb = new StringBuffer();
		sb.append(getCity());
		sb.append(", ");
		sb.append(getState());
		return sb.toString();
	}

	@Override
	public String toString() {
		StringBuffer sb = new StringBuffer();
		sb.append(getAddress1());
		sb.append("\n");
		sb.append(getAddress2());
		sb.append("\n");
		sb.append(getCity());
		sb.append(", ");
		sb.append(getState());
		sb.append(" ");
		sb.append(getPostalCode());
		sb.append("\n");
		sb.append(getCountry());
		return sb.toString();
	}

	// ----------------------------------------------------- Instance methods

	/**
	 * Set address1
	 */
	public void setAddress1(String address1) {
		this.address1 = address1;
	}

	/**
	 * Get address1
	 */
	public String getAddress1() {
		return this.address1;
	}

	/**
	 * Set address2
	 */
	public void setAddress2(String address2) {
		this.address2 = address2;
	}

	/**
	 * Get address2
	 */
	public String getAddress2() {
		return this.address2;
	}

	/**
	 * Set city
	 */
	public void setCity(String city) {
		this.city = city;
	}

	/**
	 * Get city
	 */
	public String getCity() {
		return this.city;
	}

	/**
	 * Set state 2-letter code
	 */
	public void setState(String state) {
		this.state = state;
	}

	/**
	 * Get state 2-letter code
	 */
	public String getState() {
		return this.state;
	}

	/**
	 * Set country data object
	 */
	public void setCountry(String country) {
		this.country = country;
	}

	/**
	 * Get country data object
	 */
	public String getCountry() {
		return this.country;
	}

	/**
	 * Set postal code
	 */
	public void setPostalCode(String postalCode) {
		this.postalCode = postalCode;
	}

	/**
	 * Get postal code
	 */
	public String getPostalCode() {
		return this.postalCode;
	}

	/* (non-Javadoc)
	 * @see com.toolcafe.gwt.client.model.ISearchable#getRawFtsTokens()
	 */
	@Override
	public String getRawFtsTokens() {
		
		StringBuffer sb = new StringBuffer();
		sb.append(getCity());
		sb.append(" ");
		sb.append(getState());
		sb.append(" ");
		sb.append(getPostalCode());
		
		return sb.toString();
	}

	/**
	 * Set the person
	public void setPerson(Person person) {
		this.person = person;
	}
	 */

	/**
	 * Get the person
	public Person getPerson() {
		return person;
	}
	 */

}