/**
 * Title:        user bean
 * Description:
 * Copyright:    Copyright (c)
 * Account:      ToolCafe, LLC
 * @author Rick Horowitz
 * @version 1.0
 */

package com.toolcafe.gwtstore.shared.model;

import java.util.ArrayList;
import java.util.HashSet;
import java.util.Iterator;
import java.util.Set;

import com.google.code.twig.annotation.Child;
import com.google.code.twig.annotation.Entity;
import com.google.code.twig.annotation.Id;
import com.toolcafe.gwt.client.exception.ToolCafeInputException;
import com.toolcafe.gwt.client.exception.ToolCafeValidateException;
import com.toolcafe.gwt.shared.model.ICopyable;
import com.toolcafe.gwt.shared.model.ISearchSource;
import com.toolcafe.gwt.shared.model.ISearchable;
import com.toolcafe.gwt.shared.model.IStorable;

/**
 * A person of the system
 *
 * @author Rick Horowitz
 * @version $Revision: 1.3 $ $Date: 2000/10/15 03:34:55 $
 */
@Entity(allocateIdsBy=10, kind="Person")
public class Person implements ICopyable, IStorable, ISearchable, ISearchSource {

	// ------------------------------------------------------- Statics
	
	/** Keys used to interact with fields in GWT client */
	public static enum PERSON_KEYS {
		FIRST_NAME,
		LAST_NAME,
		EMAIL_ADDR,
		PHONE_NBR
	}

	private static final long serialVersionUID = 1L;

	// ------------------------------------------------------- Instance Variables

	/** Unique id of the object - used as the object's primary key in the database */
	@Id
    private long key;
	/** The person's username/password */
	@Child
	private UserPwd userPwd = new UserPwd();
	/** The user's first name */
	private String firstName;
	/** The user's last name */
	private String lastName;
	/** The user's middle initial */
	private String middleInitial;
	/** The user's primary email mailAddress */
	private String emailAddr;
	/** Phone number */
	private String phoneNbr;
	/** Fax number */
	private String faxNbr;
	/** Salutation to use when addressing this person, e.g. Mr., Ms., Dr. */
	private String salutation;
	/** The user's mailing mailingAddress */
	@Child
	private Address mailingAddress;
	/** Full-text search tokens for this object */
	private Set<String> ftsTokens = new HashSet<String>();
    
    // The roleTypes and roleKeys arrays are parallel arrays. Their related items are in the same array positions.
    
	/** ArrayList of RoleType objects. Each instance represents a role played by this Person,
	 * and is used to identify the existence of roles in the client side of the application, 
	 * and to identify which datastore key in "roleKeys" is associated with a given RoleType */
	private ArrayList<RoleType> roleTypes = new ArrayList<RoleType>();
	/** ArrayList of database keys for the various IRoleS played by a Person. The objects associated
	 * with these keys can be decoded by calling PMFInterface.resolveClassname(). The reason we 
	 * designed it this way  */
	private ArrayList<String> roleKeys = new ArrayList<String>();
	/** ArrayList of non-persistent IRole objects. This array contains only the IRole objects 
	 * that have been obtained from the database for this Person object and cached on the client */
	@Child
	private ArrayList<IRole> roles = new ArrayList<IRole>();

	/* (non-Javadoc)
	 * @see com.toolcafe.gwt.client.model.ISerializable#makeSerializable()
	 */
	@Override
	public void loadObject() {
		mailingAddress.loadObject();
		getRoleTypes();
		getRoleKeys();
		getFtsTokens();
	}

	// ------------------------------------------------------- Constructors
	
	public Person() {
		mailingAddress = new Address();
	}
	
	public String getKey() {
		return String.valueOf(key);
	}

	public void setKey(String key) {
		this.key = Long.parseLong(key);
	}
	
	/**
	 * Set the username/password
	 * @param userPwd the userPwd to set
	 */
	public void setUserPwd(UserPwd userPwd) {
		this.userPwd = userPwd;
	}

	/**
	 * Get the username/password
	 * @return the userPwd
	 */
	public UserPwd getUserPwd() {
		return userPwd;
	}

	/**
	 * Validate the user object
	 *
	 * NOTES:
	 *   1) If the user is under a Account, allow all fields except email address to be null.
	 *   2) Do NOT validate Account here because it would get validated for each user
	 */
	public void validate() throws ToolCafeValidateException {
		// Always require an email addr for a user
		if (getEmailAddr() == null)
			throw new ToolCafeValidateException("Email mailAddress is required", null);

		// Only require the following fields if this user is not under a company
		if (getFirstName() == null)
			throw new ToolCafeValidateException("First name is required", null);
		if (getLastName() == null)
			throw new ToolCafeValidateException("Last name is required", null);
		// TODO : uncomment this when I move Address back to Person
		//if (getAddress() == null)
		//	throw new ToolCafeValidateException("Address is required", null);
		if (getPhoneNbr() == null)
			throw new ToolCafeValidateException("Phone number is required", null);
	}

	/**
	 * Require an overridden equals() method
	public boolean equals(Object obj) {
		
		if (!(obj instanceof Person))
			return false;
		
		Person that = (Person) obj;

		ClientUtil clientUtil = ClientUtil.getInstance();
		if (!clientUtil.objEquals(getFirstName(), that.getFirstName()))
			return false;
		if (!clientUtil.objEquals(getLastName(), that.getLastName()))
			return false;
		if (!clientUtil.objEquals(getEmailAddr(), that.getEmailAddr()))
			return false;
		//(TBD) - need to make PhoneNbr an object with a better equals implementation
		if (!checkPhoneNbrsEqual(that))
			return false;
		// Address test
		// TODO - uncomment this when i move Address back to Person
		// if (!appUtil.objEquals(getAddress(), that.getAddress()))
		// 	return false;
		// Passed all tests
		return true;
	}
	 */

	/**
	 * Check if the phone number equals the phone number in the other Person
	 * @param that The other person.
	 * @return True if phone numbers are equal, else false
	 */
	private boolean checkPhoneNbrsEqual(Person that) {
		// TODO : Need to check this more robustly, by squeezing out all non-digits from the strings
		return getPhoneNbr().equals(that.getPhoneNbr());
	}

	/* (non-Javadoc)
	 * @see java.lang.Object#hashCode()
	 */
	@Override
	public int hashCode() {
		final int prime = 31;
		int result = 1;
		result = prime * result + (int) (key ^ (key >>> 32));
		return result;
	}

	/* (non-Javadoc)
	 * @see java.lang.Object#equals(java.lang.Object)
	 */
	@Override
	public boolean equals(Object obj) {
		if (this == obj)
			return true;
		if (obj == null)
			return false;
		if (getClass() != obj.getClass())
			return false;
		Person other = (Person) obj;
		if (key != other.key)
			return false;
		return true;
	}

	/**
	 * Copy from source object to this object.
	 * @param person user object to copy from
	 */
	public void copyFrom(Object person, boolean deepCopy) throws ToolCafeInputException {
		if (!(person instanceof Person))
			throw new ToolCafeInputException(
				"Copy attempt failed. Requires user " + "object as source.",
				null);
		Person source = (Person) person;

		setKey(source.getKey());
		setFirstName(source.getFirstName());
		setLastName(source.getLastName());
		setMiddleInitial(source.getMiddleInitial());
		setEmailAddr(source.getEmailAddr());
		setPhoneNbr(source.getPhoneNbr());
		
		if (deepCopy) {
			// Copy mailingAddress
			if (source.getMailingAddress() != null) {
				getMailingAddress().copyFrom(source.getMailingAddress(), deepCopy);
			} else {
				setMailingAddress(new Address());
			}
			
			// Copy the UserPwd
			if (source.getUserPwd() != null) {
				getUserPwd().copyFrom(source.getUserPwd(), deepCopy);
			} else {
				setUserPwd(new UserPwd());
			}
			
			// Copy the IRole objects
			
			// First, remove IRoleS that are found in target Person, but not in the source Person
			ArrayList<IRole> sourceRoles = source.getRoles();
			ArrayList<IRole> targetRoles = new ArrayList<IRole>();
			setRoles(targetRoles);
			Iterator<IRole> targetRIter = targetRoles.iterator();
			while (targetRIter.hasNext()) {
				IRole targetR = targetRIter.next();
				if (!sourceRoles.contains(targetR))
					targetRIter.remove();
			}		
			
			// Next, copy the source IRole content into target IRole
			Iterator<IRole> sourceRIter = sourceRoles.iterator();
			while (sourceRIter.hasNext()) {
				IRole sourceR = sourceRIter.next();
				int targetIndex = targetRoles.indexOf(sourceR);
				IRole targetR = null;
				if (targetIndex >= 0) {
					targetR = targetRoles.get(targetIndex);
					targetR.copyFrom(sourceR, deepCopy);
				} else {
					// This is a new source IRole. Just add it to the target collection
					targetR = sourceR.createRole();
					targetR.copyFrom(sourceR, deepCopy); 
					targetRoles.add(targetR);
				}
				targetR.setPerson(this);
			}
			
			// Copy the role types and role keys
			setRoleTypes(new ArrayList<RoleType>(source.getRoleTypes()));
			setRoleKeys(new ArrayList<String>(source.getRoleKeys()));
			setFtsTokens(new HashSet<String>(source.getFtsTokens()));
		}
	}
	
	/**
	 * Check if the Person has the IRole, by checking if it contains the associated RoleType
	 * @param roleType The RoleType to check
	 * @return true if the RoleType is found, else false
	 */
	public boolean isRoleSet(RoleType roleType) {
		return getRoleTypes().contains(roleType);
	}
	
	/**
	 * Get the IRole instance of the specified RoleType
	 * @param roleType the RoleType to get
	 * @return The IRole instance or null if not found
	 */
	public IRole getRole(RoleType roleType) {
		int index = getRoleTypes().indexOf(roleType);
		if (index >= 0) {
			for (IRole role : getRoles()) {
				if (role.getRoleType().equals(roleType))
					return role;
			}
		}
		return null;
	}

	@Override
	public String toString() {
		StringBuffer sb = new StringBuffer();
		sb.append(getFirstName());
		sb.append(" ");
		sb.append(getLastName());
		sb.append(", ");
		sb.append(getEmailAddr());
		sb.append(", ");
		sb.append(getPhoneNbr());
		return sb.toString();
	}

	// ------------------------------------------------------ Instance methods

	/**
	 * Return the email address.
	 */
	public String getEmailAddr() {
		return (this.emailAddr);
	}
	/**
	 * Set the email address.
	 * @param emailAddr 
	 */
	public void setEmailAddr(String emailAddr) {
		this.emailAddr = emailAddr;
	}
	
	/**
	 * Get the person's full name: <first> <middle> <last>
	 * @return
	 */
	public String getFullName() {
		StringBuffer sb = new StringBuffer();
		if (getFirstName() != null) {
			sb.append(getFirstName());
			sb.append(" ");
		}
		if ((getMiddleInitial() != null) && (getMiddleInitial().length() > 0)) {
			sb.append(getMiddleInitial());
			sb.append(" ");
		} 
		if (getLastName() != null) {
			sb.append(getLastName());
		}
		return sb.toString();
	}

	/**
	 * Return the first name.
	 */
	public String getFirstName() {
		return (this.firstName);
	}
	/**
	 * Set the first name.
	 *
	 * @param firstName The new first name
	 */
	public void setFirstName(String firstName) {
		this.firstName = firstName;
	}

	/**
	 * Return the last name.
	 */
	public String getLastName() {
		return (this.lastName);
	}
	/**
	 * Set the last name.
	 *
	 * @param lastName The new last name
	 */
	public void setLastName(String lastName) {
		this.lastName = lastName;
	}

	/**
	 * Get the user's middle initial
	 * @return User's middle initial
	 */
	public String getMiddleInitial() {
		return middleInitial;
	}

	/**
	 * Set the user's middle initial
	 * @param middleInitial The middle initial to set
	 */
	public void setMiddleInitial(String middleInitial) {
		this.middleInitial = middleInitial;
	}
	public String getPhoneNbr() {
		return phoneNbr;
	}

	public void setPhoneNbr(String phoneNbr) {
		this.phoneNbr = phoneNbr;
	}

	/**
	 * Get the salutation to use when addressing this person
	 * @return Returns the salutation.
	 */
	public String getSalutation() {
		return salutation;
	}

	/**
	 * Set the salutation to use when addressing this person
	 * @param salutation The salutation to set.
	 */
	public void setSalutation(String salutation) {
		this.salutation = salutation;
	}

	/**
	 * Return the mailingAddress.
	 */
	public Address getMailingAddress() {
		return (this.mailingAddress);
	}
	/**
	 * Set the mailingAddress.
	 *
	 * @param mailingAddress The new mailingAddress
	 */
	public void setMailingAddress(Address mailingAddress) {
		this.mailingAddress = mailingAddress;
	}

	/**
	 * Set the fax number 
	 * @param faxNbr the fax nbr to set
	 */
	public void setFaxNbr(String faxNbr) {
		this.faxNbr = faxNbr;
	}

	/**
	 * Get the fax number 
	 * @return the fax nbr
	 */
	public String getFaxNbr() {
		return faxNbr;
	}

	/**
	 * Set the database keys for all IRoleS this person plays
	 * @param roleKeys the role keys to set
	 */
	public void setRoleKeys(ArrayList<String> roleKeys) {
		this.roleKeys = roleKeys;
	}

	/**
	 * Get the database keys for all IRoleS this person plays
	 * @return the role keys
	 */
	public ArrayList<String> getRoleKeys() {
		return roleKeys;
	}

	/**
	 * Set the non-persistent IRole objects for this Person
	 * @param roles the roles to set
	 */
	public void setRoles(ArrayList<IRole> roles) {
		this.roles = roles;
	}

	/**
	 * Get the non-persistent IRole objects for this Person
	 * @return the roles
	 */
	public ArrayList<IRole> getRoles() {
		return roles;
	}

	/* (non-Javadoc)
	 * @see com.toolcafe.gwt.client.model.ISearchable#getRawFtsTokens()
	 */
	@Override
	public String getRawFtsTokens() {
		
		StringBuffer sb = new StringBuffer();
		sb.append(getKey());
		sb.append(" ");
		sb.append(getFirstName());
		sb.append(" ");
		sb.append(getLastName());
		sb.append(" ");
		sb.append(getEmailAddr());
		sb.append(" ");
		sb.append(getPhoneNbr());
		sb.append(" ");
		sb.append(getFaxNbr());
		sb.append(" ");
		sb.append(getMailingAddress().getRawFtsTokens());
		
		return sb.toString();
	}

	/* (non-Javadoc)
	 * @see com.toolcafe.gwt.client.model.ISearchable#getFtsTokens()
	 */
	@Override
	public Set<String> getFtsTokens() {
		return this.ftsTokens;
	}

	/* (non-Javadoc)
	 * @see com.toolcafe.gwt.client.model.ISearchable#setFtsTokens(java.util.Set)
	 */
	@Override
	public void setFtsTokens(Set<String> ftsTokens) {
		this.ftsTokens = ftsTokens;
	}

	/**
	 * Set the role types for this Person
	 * @param roleTypes the RoleTypeS to set
	 */
	public void setRoleTypes(ArrayList<RoleType> roleTypes) {
		this.roleTypes = roleTypes;
	}

	/**
	 * Get the role types for this Person
	 * @return the RoleTypeS
	 */
	public ArrayList<RoleType> getRoleTypes() {
		return roleTypes;
	}
	
	/**
	 * Get the role's datastore key for the specified RoleType
	 * @param roleType the RoleType for which to get the datastore key
	 * @return the datastore key or null, if not found
	 */
	public String getRoleKey(RoleType roleType) {
		int index = getRoleTypes().indexOf(roleType);
		String roleKey = null;
		if (index >= 0)
			roleKey = getRoleKeys().get(index);
		return roleKey;
	}
	
	/**
	 * Add the specified IRole object to the Person. If the role type associated with the IRole
	 * already exists in the Person, replace it with the new IRole.
	 * @param role the IRole to add
	 */
	public void addRole(IRole role) {
		RoleType rt = role.getRoleType();
		
		// Remove an existing role of the same role type
		int rtIndex = getRoleTypes().indexOf(rt);
		if (rtIndex >= 0) {
			getRoleTypes().remove(rtIndex);
			getRoleKeys().remove(rtIndex);
			// Note - the IRole is not removed by index because it is not a parallel array to 'roleTypes'
			for (Iterator<IRole> roleIter = getRoles().iterator(); roleIter.hasNext(); ) {
				IRole r = roleIter.next();
				if (r.getRoleType().equals(role.getRoleType())) {
					roleIter.remove();
					break;
				}
			}
		}
		
		// Add the new role
		getRoleTypes().add(rt);
		getRoleKeys().add(role.getKey());
		getRoles().add(role);
	}
}