package com.toolcafe.gwtstore.shared.model;

/**
 * Title:        Address object
 * Description:
 * Copyright:    Copyright (c)
 * Account:      EJBPeople, Inc.
 * @author Rick Horowitz
 * @version 1.0
 */

import java.util.HashSet;

import com.google.code.twig.annotation.Entity;
import com.google.code.twig.annotation.Parent;
import com.toolcafe.gwt.client.MainMessages;
import com.toolcafe.gwt.client.exception.ToolCafeInputException;
import com.toolcafe.gwt.client.util.ClientUtil;
import com.toolcafe.gwt.client.util.ErrorResponse;
import com.toolcafe.gwt.shared.model.ICopyable;
import com.toolcafe.gwt.shared.model.ISearchSource;
import com.toolcafe.gwt.shared.model.IStorable;

@Entity(allocateIdsBy=10, kind="UserPwd")
public class UserPwd implements ICopyable, IStorable, ISearchSource {

	// ------------------------------------------------------- Statics
	/**
	 * Serializable version id When evolving this class with compatible changes,
	 * copy this version id to the new class
	 */
	static final long serialVersionUID = 100L;

	/** Keys used to interact with fields in GWT client */
	public static enum USERNAME_KEYS {
		username, password, password2
	}
	
	/** Values indicating the user's authorization */
	public static enum Authorization {
		SUPERUSER,
		ACCOUNT_ADMIN
	}

	// ----------------------------------------------------- Instance variables

	/**
	 * Unique id of the object - used as the object's primary key in the
	 * database
	 */
    private String key;
	/** Version nbr for this entity in the database */
	private Long version;
	/** The username (must be unique) - used to login to the application */
	private String username;
	/** The email address for the person */
	private String emailAddr;
	/** The password (one-way hash encrypted) - used to login to the application
	private char[] password; */
	private String password;
	/** The password2 field is used to verify that the user entered the same password twice
	 * when they set or change their password */
	private String password2;
	/** Encoded database key of parent Person */
    private String personKey;
    /** The parent Person object */
    @Parent
    private Person person;
	/** User authorization. NOTE: I tried to use an EnumSet to hold the enum values, but
	 * GAE JDO threw an exception when I tried to persist it, so I changed to HashSet. 
	private EnumSet<Authorization> authorizations = EnumSet.noneOf(Authorization.class); */
	private HashSet<Authorization> authorizations = new HashSet<Authorization>();
	
	// ----------------------------------------------------- Constructors

	public UserPwd() {		
	}

	/**
	 * Construct the UserPwd
	 * @param username 
	 * @param password
	 */
	public UserPwd(String username, String password) {

		this.username = username;
		this.password = password;
	}
	
	public String getKey() {
		return key;
	}

	public void setKey(String key) {
		this.key = key;
	}

	public Long getVersion() {
		return version;
	}

	public void setVersion(Long version) {
		this.version = version;
	}

	@Override
	public void loadObject() {
		setAuthorizations(new HashSet<Authorization>(getAuthorizations()));
	}

	/**
	 * Validate the UserPwd object
	 */
	public ErrorResponse validate() {
		ErrorResponse errResp = new ErrorResponse();
		MainMessages msgs = MainMessages.INSTANCE;
		
		// Password is not required to be entered if it has been previously entered
		boolean pwdRequired = getKey() == null;
		
		if (username == null)
			errResp.addFieldError(USERNAME_KEYS.username, msgs.usernameIsRequired());
		if (pwdRequired) {
			if (password == null)
				errResp.addFieldError(USERNAME_KEYS.password, msgs.passwordIsRequired());
			if (password2 == null)
				errResp.addFieldError(USERNAME_KEYS.password, msgs.passwordMustBeEnteredTwice());
		}
		if (password != null && !password.equals(password2))
			errResp.addFieldError(USERNAME_KEYS.password, msgs.passwordsMustMatch());
		if (pwdRequired || password.length() > 0) {
			if (password == null || password.length() < 6)
				errResp.addFieldError(USERNAME_KEYS.password, msgs.password6CharactersMin());
		}
		
		return errResp;
	}

	/**
	 * Require an overridden equals() method
	 */
	@Override
	public boolean equals(Object obj) {
		if (!(obj instanceof UserPwd))
			return false;
		UserPwd that = (UserPwd) obj;
		ClientUtil clientUtil = ClientUtil.getInstance();
		if (!clientUtil.objEquals(getUsername(), that.getUsername()))
			return false;
		if (!clientUtil.objEquals(getPassword(), that.getPassword()))
			return false;
		return true;
	}

	/**
	 * Copy from source object to this object.
	 * @param userPwd
	 *            Address object to copy from
	 */
	public void copyFrom(Object userPwd, boolean deepCopy) throws ToolCafeInputException {
		if (!(userPwd instanceof UserPwd))
			throw new ToolCafeInputException(
					"Copy attempt failed. Requires UserPwd object as source.", null);
		UserPwd source = (UserPwd) userPwd;
		if (getKey() == null)
			setKey(source.getKey());
		setPersonKey(source.getPersonKey());
		setUsername(source.getUsername());
		setEmailAddr(source.getEmailAddr());
		setAuthorizations(new HashSet<Authorization>(source.getAuthorizations()));
		
		// Only copy a non-empty password to the target because we don't want to overwrite an encrypted password
		String pwd = source.getPassword();
		if (pwd != null && pwd.length() != 0) {
			setPassword(pwd);
		}
	}

	@Override
	public String toString() {
		StringBuffer sb = new StringBuffer();
		sb.append(getUsername());
		return sb.toString();
	}

	// ----------------------------------------------------- Instance methods

	/**
	 * Return the username.
	 */
	public String getUsername() {
		return (this.username);
	}
	
	/**
	 * Set the username.
	 *
	 * @param username The new username
	 */
	public void setUsername(String username) {
		this.username = username;
	}
	
	/**
	 * Return the password.
	 */
	public String getPassword() {
		return (this.password);
	}
	
	/**
	 * Set the password.
	 *
	 * @param password The new password
	 */
	public void setPassword(String password) {
		this.password = password;
	}
	
	/**
	 * Get the password2 field
	 * @return
	 */
    public String getPassword2() {
		return password2;
	}

    /**
     * Set the password2 field
     * @param password2
     */
	public void setPassword2(String password2) {
		this.password2 = password2;
	}


	/**
	 * Set the authorizations
	 * @param authorizations the authorizations to set
	 */
	public void setAuthorizations(HashSet<Authorization> authorizations) {
		this.authorizations = authorizations;
	}

	/**
	 * Get the authorizations
	 * @return the authorizations
	 */
	public HashSet<Authorization> getAuthorizations() {
		return authorizations;
	}	
	
	public String getPersonKey() {
		return personKey;
	}

	public void setPersonKey(String parentKey) {
		this.personKey = parentKey;
	}

	/* (non-Javadoc)
	 * @see com.toolcafe.gwt.client.model.ISearchable#getRawFtsTokens()
	 */
	@Override
	public String getRawFtsTokens() {
		
		StringBuffer sb = new StringBuffer();
		sb.append(getUsername());
		sb.append(" ");
		sb.append(getPersonKey());
		
		return sb.toString();
	}

	/**
	 * Return the email address.
	 */
	public String getEmailAddr() {
		return (this.emailAddr);
	}
	
	/**
	 * Set the email address.
	 * @param emailAddr 
	 */
	public void setEmailAddr(String emailAddr) {
		this.emailAddr = emailAddr;
	}

	/**
	 * Set the parent Person object
	 * @param person the person to set
	 */
	public void setPerson(Person person) {
		this.person = person;
	}

	/**
	 * Get the parent Person object
	 * @return the person
	 */
	public Person getPerson() {
		return person;
	}
}